<?php
/*##################################################
 *                       SeriesFormController.class.php
 *                            -------------------
 *   begin                : February 27, 2013
 *   copyright            : (C) 2014 Root
 *   email                : contact@graphismeo.ovh
 *
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

/**
 * @author Patrick DUBEAU <daaxwizeman@gmail.com>
 */
class SeriesFormController extends ModuleController
{
	private $tpl;
	private $lang;
	private $form;
	private $submit_button;
	private $serie;

	public function execute(HTTPRequestCustom $request)
	{
		$this->init();
		$this->check_authorizations();
		$this->build_form($request);
                
		if ($this->submit_button->has_been_submited() && $this->form->validate())
		{
			$this->save();
			$this->redirect();
		}
		
		$this->tpl->put('FORM', $this->form->display());
		return $this->build_response($this->tpl);
	}

	private function init()
	{
		$this->lang = LangLoader::get('common', 'series');
		$this->tpl = new FileTemplate('series/SeriesFormController.tpl');
		$this->tpl->add_lang($this->lang);
	}
	
	private function build_form(HTTPRequestCustom $request)
	{
		$common_lang = LangLoader::get('common');
		
		$form = new HTMLForm(__CLASS__);
		
		$fieldset = new FormFieldsetHTML('series', $this->lang['series']);
		$form->add_fieldset($fieldset);

		$fieldset->add_field(new FormFieldTextEditor('title', $this->lang['series.form.title'], $this->get_serie()->get_title(), 
			array('required' => true)
		));

		if (!$this->is_contributor_member())
		{
			$fieldset->add_field(new FormFieldCheckbox('personalize_rewrited_title', $this->lang['series.form.rewrited_title.personalize'], $this->get_serie()->rewrited_title_is_personalized(),
				array('events' => array('click' =>'
					if (HTMLForms.getField("personalize_rewrited_title").getValue()) {
						HTMLForms.getField("rewrited_title").enable();
					} else { 
						HTMLForms.getField("rewrited_title").disable();
					}'
				))
			));

			$fieldset->add_field(new FormFieldTextEditor('rewrited_title', $this->lang['series.form.rewrited_title'], $this->get_serie()->get_rewrited_title(),
				array('description' => $this->lang['series.form.rewrited_title.description'],
				      'hidden' => !$this->get_serie()->rewrited_title_is_personalized()),
				array(new FormFieldConstraintRegex('`^[a-z0-9\-]+$`i'))
			));
		}

		$search_category_children_options = new SearchCategoryChildrensOptions();
		$search_category_children_options->add_authorizations_bits(Category::READ_AUTHORIZATIONS);
		$search_category_children_options->add_authorizations_bits(Category::CONTRIBUTION_AUTHORIZATIONS);
		$fieldset->add_field(SeriesService::get_categories_manager()->get_select_categories_form_field('id_category', LangLoader::get_message('category', 'categories-common'), $this->get_serie()->get_id_category(), $search_category_children_options));
		

		
		$fieldset->add_field(new FormFieldRichTextEditor('description', StringVars::replace_vars($this->lang['series.form.description'],
			array('number' =>SeriesConfig::load()->get_number_character_to_cut())), $this->get_serie()->get_description(),
			array('rows' => 3, 'hidden' => !$this->get_serie()->get_description_enabled())
		));
		
		$fieldset->add_field(new FormFieldRichTextEditor('contents', $common_lang['form.contents'], $this->get_serie()->get_contents(),
			array('rows' => 15, 'required' => true)
		));
		
		$onclick_action = 'javascript:bbcode_page();';
		$fieldset->add_field(new FormFieldActionLink('add_page', $this->lang['series.form.add_page'] , $onclick_action, PATH_TO_ROOT . '/series/templates/images/pagebreak.png'));
		$fieldset->add_field(new FormFieldSimpleSelectChoice('statut', $this->lang['series.statut'],  $this->get_serie()->get_annee(),
			array(
				new FormFieldSelectChoiceOption($this->lang['series.statut.production'], $this->lang['series.statut.production']),
				new FormFieldSelectChoiceOption($this->lang['series.statut.stop'], $this->lang['series.statut.stop']),
			), array('required' => false)
		));
		$fieldset->add_field(new FormFieldTextEditor('duree', $this->lang['series.duree'], $this->get_serie()->get_duree()));
		$fieldset->add_field(new FormFieldTextEditor('acteurs', $this->lang['series.acteurs'], $this->get_serie()->get_acteurs()));
		$fieldset->add_field(new FormFieldTextEditor('annee', $this->lang['series.annee'], $this->get_serie()->get_annee()));
		$fieldset->add_field(new FormFieldTextEditor('episode1', $this->lang['series.episode1'], $this->get_serie()->get_episode1()));
		$fieldset->add_field(new FormFieldTextEditor('episode2', $this->lang['series.episode2'], $this->get_serie()->get_episode2()));
		$fieldset->add_field(new FormFieldTextEditor('episode3', $this->lang['series.episode3'], $this->get_serie()->get_episode3()));
		$fieldset->add_field(new FormFieldTextEditor('episode4', $this->lang['series.episode4'], $this->get_serie()->get_episode4()));
		$fieldset->add_field(new FormFieldTextEditor('episode5', $this->lang['series.episode5'], $this->get_serie()->get_episode5()));
		$fieldset->add_field(new FormFieldTextEditor('episode6', $this->lang['series.episode6'], $this->get_serie()->get_episode6()));
		$fieldset->add_field(new FormFieldTextEditor('episode7', $this->lang['series.episode7'], $this->get_serie()->get_episode7()));
		$fieldset->add_field(new FormFieldTextEditor('episode8', $this->lang['series.episode8'], $this->get_serie()->get_episode8()));
		$fieldset->add_field(new FormFieldTextEditor('episode9', $this->lang['series.episode9'], $this->get_serie()->get_episode9()));
		$fieldset->add_field(new FormFieldTextEditor('episode10', $this->lang['series.episode10'], $this->get_serie()->get_episode10()));
		$fieldset->add_field(new FormFieldTextEditor('episode11', $this->lang['series.episode11'], $this->get_serie()->get_episode11()));
		$fieldset->add_field(new FormFieldTextEditor('episode12', $this->lang['series.episode12'], $this->get_serie()->get_episode12()));
		$fieldset->add_field(new FormFieldTextEditor('episode13', $this->lang['series.episode13'], $this->get_serie()->get_episode13()));
		$fieldset->add_field(new FormFieldTextEditor('episode14', $this->lang['series.episode14'], $this->get_serie()->get_episode14()));
		$fieldset->add_field(new FormFieldTextEditor('episode15', $this->lang['series.episode15'], $this->get_serie()->get_episode15()));
		$fieldset->add_field(new FormFieldTextEditor('episode16', $this->lang['series.episode16'], $this->get_serie()->get_episode16()));
		$fieldset->add_field(new FormFieldTextEditor('episode17', $this->lang['series.episode17'], $this->get_serie()->get_episode17()));
		$fieldset->add_field(new FormFieldTextEditor('episode18', $this->lang['series.episode18'], $this->get_serie()->get_episode18()));
		$fieldset->add_field(new FormFieldTextEditor('episode19', $this->lang['series.episode19'], $this->get_serie()->get_episode19()));
		$fieldset->add_field(new FormFieldTextEditor('episode20', $this->lang['series.episode20'], $this->get_serie()->get_episode20()));
		$fieldset->add_field(new FormFieldTextEditor('episode21', $this->lang['series.episode21'], $this->get_serie()->get_episode21()));
		$fieldset->add_field(new FormFieldTextEditor('episode22', $this->lang['series.episode22'], $this->get_serie()->get_episode22()));
		$fieldset->add_field(new FormFieldTextEditor('episode23', $this->lang['series.episode23'], $this->get_serie()->get_episode23()));
		$fieldset->add_field(new FormFieldTextEditor('episode24', $this->lang['series.episode24'], $this->get_serie()->get_episode24()));
		$fieldset->add_field(new FormFieldTextEditor('episode25', $this->lang['series.episode25'], $this->get_serie()->get_episode25()));
		$fieldset->add_field(new FormFieldTextEditor('episode26', $this->lang['series.episode26'], $this->get_serie()->get_episode26()));
		$fieldset->add_field(new FormFieldTextEditor('episode27', $this->lang['series.episode27'], $this->get_serie()->get_episode27()));
		$fieldset->add_field(new FormFieldTextEditor('episode28', $this->lang['series.episode28'], $this->get_serie()->get_episode28()));
		$fieldset->add_field(new FormFieldTextEditor('episode29', $this->lang['series.episode29'], $this->get_serie()->get_episode29()));
		$fieldset->add_field(new FormFieldTextEditor('episode30', $this->lang['series.episode30'], $this->get_serie()->get_episode30()));
		$fieldset->add_field(new FormFieldTextEditor('episode31', $this->lang['series.episode31'], $this->get_serie()->get_episode31()));
		$fieldset->add_field(new FormFieldTextEditor('episode32', $this->lang['series.episode32'], $this->get_serie()->get_episode32()));
		$fieldset->add_field(new FormFieldTextEditor('episode33', $this->lang['series.episode33'], $this->get_serie()->get_episode33()));
		$fieldset->add_field(new FormFieldTextEditor('episode34', $this->lang['series.episode34'], $this->get_serie()->get_episode34()));
		$fieldset->add_field(new FormFieldTextEditor('episode35', $this->lang['series.episode35'], $this->get_serie()->get_episode35()));
		$fieldset->add_field(new FormFieldTextEditor('episode36', $this->lang['series.episode36'], $this->get_serie()->get_episode36()));
		$fieldset->add_field(new FormFieldTextEditor('episode37', $this->lang['series.episode37'], $this->get_serie()->get_episode37()));
		$fieldset->add_field(new FormFieldTextEditor('episode38', $this->lang['series.episode38'], $this->get_serie()->get_episode38()));
		$fieldset->add_field(new FormFieldTextEditor('episode39', $this->lang['series.episode39'], $this->get_serie()->get_episode39()));
		$fieldset->add_field(new FormFieldTextEditor('episode40', $this->lang['series.episode40'], $this->get_serie()->get_episode40()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr1', $this->lang['series.episodevostfr1'], $this->get_serie()->get_episodevostfr1()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr2', $this->lang['series.episodevostfr2'], $this->get_serie()->get_episodevostfr2()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr3', $this->lang['series.episodevostfr3'], $this->get_serie()->get_episodevostfr3()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr4', $this->lang['series.episodevostfr4'], $this->get_serie()->get_episodevostfr4()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr5', $this->lang['series.episodevostfr5'], $this->get_serie()->get_episodevostfr5()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr6', $this->lang['series.episodevostfr6'], $this->get_serie()->get_episodevostfr6()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr7', $this->lang['series.episodevostfr7'], $this->get_serie()->get_episodevostfr7()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr8', $this->lang['series.episodevostfr8'], $this->get_serie()->get_episodevostfr8()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr9', $this->lang['series.episodevostfr9'], $this->get_serie()->get_episodevostfr9()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr10', $this->lang['series.episodevostfr10'], $this->get_serie()->get_episodevostfr10()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr11', $this->lang['series.episodevostfr11'], $this->get_serie()->get_episodevostfr11()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr12', $this->lang['series.episodevostfr12'], $this->get_serie()->get_episodevostfr12()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr13', $this->lang['series.episodevostfr13'], $this->get_serie()->get_episodevostfr13()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr14', $this->lang['series.episodevostfr14'], $this->get_serie()->get_episodevostfr14()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr15', $this->lang['series.episodevostfr15'], $this->get_serie()->get_episodevostfr15()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr16', $this->lang['series.episodevostfr16'], $this->get_serie()->get_episodevostfr16()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr17', $this->lang['series.episodevostfr17'], $this->get_serie()->get_episodevostfr17()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr18', $this->lang['series.episodevostfr18'], $this->get_serie()->get_episodevostfr18()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr19', $this->lang['series.episodevostfr19'], $this->get_serie()->get_episodevostfr19()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr20', $this->lang['series.episodevostfr20'], $this->get_serie()->get_episodevostfr20()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr21', $this->lang['series.episodevostfr21'], $this->get_serie()->get_episodevostfr21()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr22', $this->lang['series.episodevostfr22'], $this->get_serie()->get_episodevostfr22()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr23', $this->lang['series.episodevostfr23'], $this->get_serie()->get_episodevostfr23()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr24', $this->lang['series.episodevostfr24'], $this->get_serie()->get_episodevostfr24()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr25', $this->lang['series.episodevostfr25'], $this->get_serie()->get_episodevostfr25()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr26', $this->lang['series.episodevostfr26'], $this->get_serie()->get_episodevostfr26()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr27', $this->lang['series.episodevostfr27'], $this->get_serie()->get_episodevostfr27()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr28', $this->lang['series.episodevostfr28'], $this->get_serie()->get_episodevostfr28()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr29', $this->lang['series.episodevostfr29'], $this->get_serie()->get_episodevostfr29()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr30', $this->lang['series.episodevostfr30'], $this->get_serie()->get_episodevostfr30()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr31', $this->lang['series.episodevostfr31'], $this->get_serie()->get_episodevostfr31()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr32', $this->lang['series.episodevostfr32'], $this->get_serie()->get_episodevostfr32()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr33', $this->lang['series.episodevostfr33'], $this->get_serie()->get_episodevostfr33()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr34', $this->lang['series.episodevostfr34'], $this->get_serie()->get_episodevostfr34()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr35', $this->lang['series.episodevostfr35'], $this->get_serie()->get_episodevostfr35()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr36', $this->lang['series.episodevostfr36'], $this->get_serie()->get_episodevostfr36()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr37', $this->lang['series.episodevostfr37'], $this->get_serie()->get_episodevostfr37()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr38', $this->lang['series.episodevostfr38'], $this->get_serie()->get_episodevostfr38()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr39', $this->lang['series.episodevostfr39'], $this->get_serie()->get_episodevostfr39()));
		$fieldset->add_field(new FormFieldTextEditor('episodevostfr40', $this->lang['series.episodevostfr40'], $this->get_serie()->get_episodevostfr40()));

		$other_fieldset = new FormFieldsetHTML('other', $common_lang['form.other']);
		$form->add_fieldset($other_fieldset);


		
		$other_fieldset->add_field(new FormFieldUploadFile('picture', $this->lang['series.form.picture'], $this->get_serie()->get_picture()->relative(), array('description' => $this->lang['series.form.picture.description'])));
		$other_fieldset->add_field(SeriesService::get_keywords_manager()->get_form_field($this->get_serie()->get_id(), 'keywords', $common_lang['form.keywords'],  
			array('description' => $this->lang['series.form.keywords.description'])
		));


		if (!$this->is_contributor_member())
		{
			$publication_fieldset = new FormFieldsetHTML('publication', $common_lang['form.approbation']);
			$form->add_fieldset($publication_fieldset);

			$publication_fieldset->add_field(new FormFieldDateTime('date_created', $common_lang['form.date.creation'], $this->get_serie()->get_date_created()));

			$publication_fieldset->add_field(new FormFieldSimpleSelectChoice('publishing_state', $common_lang['form.approbation'], $this->get_serie()->get_publishing_state(),
				array(
					new FormFieldSelectChoiceOption($common_lang['form.approbation.not'], Serie::NOT_PUBLISHED),
					new FormFieldSelectChoiceOption($common_lang['form.approbation.now'], Serie::PUBLISHED_NOW),
					new FormFieldSelectChoiceOption($common_lang['form.approbation.date'], Serie::PUBLISHED_DATE),
				),
				array('events' => array('change' => '
				if (HTMLForms.getField("publishing_state").getValue() == 2) {
					$("'.__CLASS__.'_publishing_start_date_field").appear();
					HTMLForms.getField("end_date_enable").enable();
				} else { 
					$("'.__CLASS__.'_publishing_start_date_field").fade();
					HTMLForms.getField("end_date_enable").disable();
				}'))
			));

			$publication_fieldset->add_field(new FormFieldDateTime('publishing_start_date', $common_lang['form.date.start'], 
				($this->get_serie()->get_publishing_start_date() === null ? new Date() : $this->get_serie()->get_publishing_start_date()), 
				array('hidden' => ($this->get_serie()->get_publishing_state() != Serie::PUBLISHED_DATE))
			));

			$publication_fieldset->add_field(new FormFieldCheckbox('end_date_enable', $common_lang['form.date.end.enable'], $this->get_serie()->end_date_enabled(), 
				array('hidden' => ($this->get_serie()->get_publishing_state() != Serie::PUBLISHED_DATE),
					'events' => array('click' => '
						if (HTMLForms.getField("end_date_enable").getValue()) {
							HTMLForms.getField("publishing_end_date").enable();
						} else { 
							HTMLForms.getField("publishing_end_date").disable();
						}'
				))
			));

			$publication_fieldset->add_field(new FormFieldDateTime('publishing_end_date', $common_lang['form.date.end'], 
				($this->get_serie()->get_publishing_end_date() === null ? new date() : $this->get_serie()->get_publishing_end_date()), 
				array('hidden' => !$this->get_serie()->end_date_enabled())
			));
		}

		$this->build_contribution_fieldset($form);
		
		$this->submit_button = new FormButtonDefaultSubmit();
		$form->add_button($this->submit_button);
		$form->add_button(new FormButtonReset());
		
		$this->form = $form;
		
		// Positionnement à la bonne page quand on édite un serie avec plusieurs pages
		if ($this->get_serie()->get_id() !== null)
		{
			$current_page = $request->get_getstring('page', '');
			
			$this->tpl->put('C_PAGE', !empty($current_page));
			
			if (!empty($current_page))
			{
				$serie_contents = $this->serie->get_contents();
				
				//If serie doesn't begin with a page, we insert one
				if (substr(trim($serie_contents), 0, 6) != '[page]')
				{
					$serie_contents = '[page]&nbsp;[/page]' . $serie_contents;
				}
				
				//Removing [page] bbcode
				$serie_contents_clean = preg_split('`\[page\].+\[/page\](.*)`Us', $serie_contents, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
				
				//Retrieving pages 
				preg_match_all('`\[page\]([^[]+)\[/page\]`U', $serie_contents, $array_page);

				$page_name = (isset($array_page[1][$current_page-1]) && $array_page[1][$current_page-1] != '&nbsp;') ? $array_page[1][($current_page-1)] : '';
				
				$this->tpl->put('PAGE', TextHelper::to_js_string($page_name));
			}
		}
	}

	private function build_contribution_fieldset($form)
	{
		if ($this->get_serie()->get_id() === null && $this->is_contributor_member())
		{
			$fieldset = new FormFieldsetHTML('contribution', LangLoader::get_message('contribution', 'user-common'));
			$fieldset->set_description(MessageHelper::display(LangLoader::get_message('contribution.explain', 'user-common'), MessageHelper::WARNING)->render());
			$form->add_fieldset($fieldset);

			$fieldset->add_field(new FormFieldRichTextEditor('contribution_description', LangLoader::get_message('contribution.description', 'user-common'), '', 
				array('description' => LangLoader::get_message('contribution.description.explain', 'user-common'))));
		}
	}

	private function is_contributor_member()
	{
		return (!SeriesAuthorizationsService::check_authorizations()->write() && SeriesAuthorizationsService::check_authorizations()->contribution());
	}

	private function get_serie()
	{
		if ($this->serie === null)
		{
			$id = AppContext::get_request()->get_getint('id', 0);
			if (!empty($id))
			{
				try
				{
					$this->serie = SeriesService::get_serie('WHERE series.id=:id', array('id' => $id));
				}
				catch(RowNotFoundException $e)
				{
					$error_controller = PHPBoostErrors::unexisting_page();
					DispatchManager::redirect($error_controller);
				}
			}
			else
			{
				$this->serie = new Serie();
				$this->serie->init_default_properties(AppContext::get_request()->get_getint('id_category', Category::ROOT_CATEGORY));
			}
		}
		return $this->serie;
	}

	private function check_authorizations()
	{
		$serie = $this->get_serie();
                
		if ($serie->get_id() === null)
		{
			if (!$serie->is_authorized_add())
			{
				$error_controller = PHPBoostErrors::user_not_authorized();
				DispatchManager::redirect($error_controller);
			}
		}
		else
		{
			if (!$serie->is_authorized_edit())
			{
				$error_controller = PHPBoostErrors::user_not_authorized();
				DispatchManager::redirect($error_controller);
			}
		}
		if (AppContext::get_current_user()->is_readonly())
		{
			$controller = PHPBoostErrors::user_in_read_only();
			DispatchManager::redirect($controller);
		}
	}

	private function save()
	{
		$serie = $this->get_serie();
		
		$serie->set_title($this->form->get_value('title'));
                $serie->set_id_category($this->form->get_value('id_category')->get_raw_value());
		$serie->set_description(($this->form->get_value('enable_description') ? $this->form->get_value('description') : ''));
		$serie->set_contents($this->form->get_value('contents'));
		$serie->set_episode1($this->form->get_value('episode1'));
		$serie->set_episode2($this->form->get_value('episode2'));
		$serie->set_episode3($this->form->get_value('episode3'));
		$serie->set_episode4($this->form->get_value('episode4'));
		$serie->set_episode5($this->form->get_value('episode5'));
		$serie->set_episode6($this->form->get_value('episode6'));
		$serie->set_episode7($this->form->get_value('episode7'));
		$serie->set_episode8($this->form->get_value('episode8'));
		$serie->set_episode9($this->form->get_value('episode9'));
		$serie->set_episode10($this->form->get_value('episode10'));
		$serie->set_episode11($this->form->get_value('episode11'));
		$serie->set_episode12($this->form->get_value('episode12'));
		$serie->set_episode13($this->form->get_value('episode13'));
		$serie->set_episode14($this->form->get_value('episode14'));
		$serie->set_episode15($this->form->get_value('episode15'));
		$serie->set_episode16($this->form->get_value('episode16'));
		$serie->set_episode17($this->form->get_value('episode17'));
		$serie->set_episode18($this->form->get_value('episode18'));
		$serie->set_episode19($this->form->get_value('episode19'));
		$serie->set_episode20($this->form->get_value('episode20'));
		$serie->set_episode21($this->form->get_value('episode21'));
		$serie->set_episode22($this->form->get_value('episode22'));
		$serie->set_episode23($this->form->get_value('episode23'));
		$serie->set_episode24($this->form->get_value('episode24'));
		$serie->set_episode25($this->form->get_value('episode25'));
		$serie->set_episode26($this->form->get_value('episode26'));
		$serie->set_episode27($this->form->get_value('episode27'));
		$serie->set_episode28($this->form->get_value('episode28'));
		$serie->set_episode29($this->form->get_value('episode29'));
		$serie->set_episode30($this->form->get_value('episode30'));
		$serie->set_episode31($this->form->get_value('episode31'));
		$serie->set_episode32($this->form->get_value('episode32'));
		$serie->set_episode33($this->form->get_value('episode33'));
		$serie->set_episode34($this->form->get_value('episode34'));
		$serie->set_episode35($this->form->get_value('episode35'));
		$serie->set_episode36($this->form->get_value('episode36'));
		$serie->set_episode37($this->form->get_value('episode37'));
		$serie->set_episode38($this->form->get_value('episode38'));
		$serie->set_episode39($this->form->get_value('episode39'));
		$serie->set_episode40($this->form->get_value('episode40'));
		$serie->set_episodevostfr1($this->form->get_value('episodevostfr1'));
		$serie->set_episodevostfr2($this->form->get_value('episodevostfr2'));
		$serie->set_episodevostfr3($this->form->get_value('episodevostfr3'));
		$serie->set_episodevostfr4($this->form->get_value('episodevostfr4'));
		$serie->set_episodevostfr5($this->form->get_value('episodevostfr5'));
		$serie->set_episodevostfr6($this->form->get_value('episodevostfr6'));
		$serie->set_episodevostfr7($this->form->get_value('episodevostfr7'));
		$serie->set_episodevostfr8($this->form->get_value('episodevostfr8'));
		$serie->set_episodevostfr9($this->form->get_value('episodevostfr9'));
		$serie->set_episodevostfr10($this->form->get_value('episodevostfr10'));
		$serie->set_episodevostfr11($this->form->get_value('episodevostfr11'));
		$serie->set_episodevostfr12($this->form->get_value('episodevostfr12'));
		$serie->set_episodevostfr13($this->form->get_value('episodevostfr13'));
		$serie->set_episodevostfr14($this->form->get_value('episodevostfr14'));
		$serie->set_episodevostfr15($this->form->get_value('episodevostfr15'));
		$serie->set_episodevostfr16($this->form->get_value('episodevostfr16'));
		$serie->set_episodevostfr17($this->form->get_value('episodevostfr17'));
		$serie->set_episodevostfr18($this->form->get_value('episodevostfr18'));
		$serie->set_episodevostfr19($this->form->get_value('episodevostfr19'));
		$serie->set_episodevostfr20($this->form->get_value('episodevostfr20'));
		$serie->set_episodevostfr21($this->form->get_value('episodevostfr21'));
		$serie->set_episodevostfr22($this->form->get_value('episodevostfr22'));
		$serie->set_episodevostfr23($this->form->get_value('episodevostfr23'));
		$serie->set_episodevostfr24($this->form->get_value('episodevostfr24'));
		$serie->set_episodevostfr25($this->form->get_value('episodevostfr25'));
		$serie->set_episodevostfr26($this->form->get_value('episodevostfr26'));
		$serie->set_episodevostfr27($this->form->get_value('episodevostfr27'));
		$serie->set_episodevostfr28($this->form->get_value('episodevostfr28'));
		$serie->set_episodevostfr29($this->form->get_value('episodevostfr29'));
		$serie->set_episodevostfr30($this->form->get_value('episodevostfr30'));
		$serie->set_episodevostfr31($this->form->get_value('episodevostfr31'));
		$serie->set_episodevostfr32($this->form->get_value('episodevostfr32'));
		$serie->set_episodevostfr33($this->form->get_value('episodevostfr33'));
		$serie->set_episodevostfr34($this->form->get_value('episodevostfr34'));
		$serie->set_episodevostfr35($this->form->get_value('episodevostfr35'));
		$serie->set_episodevostfr36($this->form->get_value('episodevostfr36'));
		$serie->set_episodevostfr37($this->form->get_value('episodevostfr37'));
		$serie->set_episodevostfr38($this->form->get_value('episodevostfr38'));
		$serie->set_episodevostfr39($this->form->get_value('episodevostfr39'));
		$serie->set_episodevostfr40($this->form->get_value('episodevostfr40'));
		$serie->set_statut($this->form->get_value('statut')->get_raw_value());
		$serie->set_acteurs($this->form->get_value('acteurs'));
		$serie->set_duree($this->form->get_value('duree'));
		$serie->set_annee($this->form->get_value('annee'));
		$author_name_displayed = $this->form->get_value('author_name_displayed') ? $this->form->get_value('author_name_displayed') : Serie::AUTHOR_NAME_NOTDISPLAYED;
		$serie->set_author_name_displayed($author_name_displayed);
		$notation_enabled = $this->form->get_value('notation_enabled') ? $this->form->get_value('notation_enabled') : Serie::NOTATION_DISABLED;
		$serie->set_notation_enabled($notation_enabled);
		$serie->set_picture(new Url($this->form->get_value('picture')));
		
		
		if ($serie->get_id() === null && $this->is_contributor_member())
		{
			$serie->set_rewrited_title(Url::encode_rewrite($serie->get_title()));
			$serie->set_publishing_state(Serie::NOT_PUBLISHED);
			$serie->set_date_created(new Date());
			$serie->clean_publishing_start_and_end_date();
		}
		else
		{
			$serie->set_date_created($this->form->get_value('date_created'));
			$rewrited_title = $this->form->get_value('rewrited_title', '');
			$rewrited_title = $this->form->get_value('personalize_rewrited_title') && !empty($rewrited_title) ? $rewrited_title : Url::encode_rewrite($serie->get_title());
			$serie->set_rewrited_title($rewrited_title);

			$serie->set_publishing_state($this->form->get_value('publishing_state')->get_raw_value());
			if ($serie->get_publishing_state() == Serie::PUBLISHED_DATE)
			{
				$serie->set_publishing_start_date($this->form->get_value('publishing_start_date'));

				if ($this->form->get_value('end_date_enable'))
				{
					$serie->set_publishing_end_date($this->form->get_value('publishing_end_date'));
				}
				else
				{
					$serie->clean_publishing_end_date();
				}
			}
			else
			{
				$serie->clean_publishing_start_and_end_date();
			}
		}
		
		if ($serie->get_id() === null)
		{
			$serie->set_author_user(AppContext::get_current_user());
			$id_serie = SeriesService::add($serie);
		}
		else
		{
			$now = new Date();
			$serie->set_date_updated($now);
			$id_serie = $serie->get_id();
			SeriesService::update($serie);
		}

		$this->contribution_actions($serie, $id_serie);
		SeriesService::get_keywords_manager()->put_relations($id_serie, $this->form->get_value('keywords'));
		Feed::clear_cache('series');
	}

	private function contribution_actions(Serie $serie, $id_serie)
	{
		if ($serie->get_id() === null)
		{
			if ($this->is_contributor_member())
			{
				$contribution = new Contribution();
				$contribution->set_id_in_module($id_serie);
				$contribution->set_description(stripslashes($this->form->get_value('contribution_description')));
				$contribution->set_entitled(StringVars::replace_vars($this->lang['series.form.contribution_entitled'], array('module_name' => $this->lang['series'], 'title' => $serie->get_title())));
				$contribution->set_fixing_url(SeriesUrlBuilder::edit_serie($id_serie)->relative());
				$contribution->set_poster_id(AppContext::get_current_user()->get_attribute('user_id'));
				$contribution->set_module('series');
				$contribution->set_auth(
					Authorizations::capture_and_shift_bit_auth(
						SeriesService::get_categories_manager()->get_heritated_authorizations($serie->get_id_category(), Category::MODERATION_AUTHORIZATIONS, Authorizations::AUTH_CHILD_PRIORITY),
						Category::MODERATION_AUTHORIZATIONS, Contribution::CONTRIBUTION_AUTH_BIT
					)
				);
				ContributionService::save_contribution($contribution);
			}
		}
		else
		{
			$corresponding_contributions = ContributionService::find_by_criteria('series', $id_serie);
			if (count($corresponding_contributions) > 0)
			{
				$serie_contribution = $corresponding_contributions[0];
				$serie_contribution->set_status(Event::EVENT_STATUS_PROCESSED);

				ContributionService::save_contribution($serie_contribution);
			}
		}
		$serie->set_id($id_serie);
	}
	
	private function redirect()
	{
		$serie = $this->get_serie();
		$category = SeriesService::get_categories_manager()->get_categories_cache()->get_category($serie->get_id_category());

		if ($serie->get_id() === null && $this->is_contributor_member() && !$serie->is_published())
		{
			AppContext::get_response()->redirect(UserUrlBuilder::contribution_success());
		}
		elseif ($serie->is_published())
		{
			AppContext::get_response()->redirect(SeriesUrlBuilder::display_serie($category->get_id(), $category->get_rewrited_name(), $serie->get_id(), $serie->get_rewrited_title()));
		}
		else
		{
			AppContext::get_response()->redirect(SeriesUrlBuilder::display_pending_series());
		}
	}

	private function build_response(View $tpl)
	{
		$serie = $this->get_serie();
		
		$response = new SiteDisplayResponse($tpl);
		$graphical_environment = $response->get_graphical_environment();
		
		$breadcrumb = $graphical_environment->get_breadcrumb();
		$breadcrumb->add($this->lang['series'], SeriesUrlBuilder::home());

		if ($serie->get_id() === null)
		{
			$breadcrumb->add($this->lang['series.add'], SeriesUrlBuilder::add_serie($serie->get_id_category()));
			$graphical_environment->set_page_title($this->lang['series.add']);
			$graphical_environment->get_seo_meta_data()->set_description($this->lang['series.add']);
			$graphical_environment->get_seo_meta_data()->set_canonical_url(SeriesUrlBuilder::add_serie($serie->get_id_category()));
		}
		else
		{
			$categories = array_reverse(SeriesService::get_categories_manager()->get_parents($serie->get_id_category(), true));
			foreach ($categories as $id => $category)
			{
				if ($category->get_id() != Category::ROOT_CATEGORY)
					$breadcrumb->add($category->get_name(), SeriesUrlBuilder::display_category($category->get_id(), $category->get_rewrited_name()));
			}
			$breadcrumb->add($serie->get_title(), SeriesUrlBuilder::display_serie($category->get_id(), $category->get_rewrited_name(), $serie->get_id(), $serie->get_rewrited_title()));

			$breadcrumb->add($this->lang['series.edit'], SeriesUrlBuilder::edit_serie($serie->get_id()));
			$graphical_environment->set_page_title($this->lang['series.edit']);
			$graphical_environment->get_seo_meta_data()->set_description($this->lang['series.edit']);
			$graphical_environment->get_seo_meta_data()->set_canonical_url(SeriesUrlBuilder::edit_serie($serie->get_id()));
		}

		return $response;
	}
}
?>
